from enum import Enum

from ..settings import settings


class Services(str, Enum):
    dict = settings.PROJECT_HOST_SERVICE_DICT
    patient = settings.PROJECT_HOST_SERVICE_PATIENTS
    practice = settings.PROJECT_HOST_SERVICE_PRACTICES
    ml = settings.PROJECT_HOST_SERVICE_ML



import uuid

from fastapi import Depends

from ..enums import Role
from ..utils.aws_cognito import AWSCognitoService, get_aws_cognito
from ..dto import UserDTO
from ..utils.bearer import get_token
from ..role_hierarchy import ROLES_PRACTICE


def require_user(
        token: str = Depends(get_token),
        cognito_service: AWSCognitoService = Depends(get_aws_cognito)
) -> UserDTO:
    user_data = cognito_service.get_current_user(token)

    user_attributes = cognito_service.get_user_attributes(token)
    practice_id_str = user_attributes.get("custom:practice_id")
    practice_id = uuid.UUID(practice_id_str) if practice_id_str else None
    user_data["practice_id"] = practice_id
    user_data["is_admin"] = Role.ADMIN in user_data["roles"]
    user_data["is_practice_dependant"] = any(
        role in ROLES_PRACTICE for role in user_data["roles"]
    )

    return UserDTO(**user_data)



from fastapi import Depends, HTTPException, status

from a360_security.enums.role import Role
from ..role_hierarchy import has_role
from ..utils.aws_cognito import AWSCognitoService, get_aws_cognito
from ..utils.bearer import get_token


async def get_current_user_roles(
        token: str = Depends(get_token),
        cognito_service: AWSCognitoService = Depends(get_aws_cognito)
) -> list[str]:
    user_data = cognito_service.get_current_user(token)
    return user_data.get("roles", [])


def require_role(*required_roles: Role):
    def role_checker(user_roles: list[str] = Depends(get_current_user_roles)):
        if not any(has_role(user_roles, role) for role in required_roles):
            roles_str = ", ".join([role.value for role in required_roles])
            raise HTTPException(
                status_code=status.HTTP_403_FORBIDDEN,
                detail=f"Insufficient permissions: requires one of {roles_str}"
            )
    return role_checker



from fastapi import Depends

from .services import DictionaryService, PracticeService
from .utils.service_provider import ServiceProvider, get_service_provider


def get_dictionary_service(service_provider: ServiceProvider = Depends(
        get_service_provider)) -> DictionaryService:
    return DictionaryService(service_provider=service_provider)


def get_practice_service(service_provider: ServiceProvider = Depends(
        get_service_provider)) -> PracticeService:
    return PracticeService(service_provider=service_provider)



import uuid
from typing import Optional

from pydantic import BaseModel, EmailStr


class UserDTO(BaseModel):
    username: str
    email: Optional[EmailStr]
    sub: uuid.UUID
    roles: list[str]
    practice_id: Optional[uuid.UUID]
    is_admin: bool
    is_practice_dependant: bool
    is_active: bool

    class Config:
        from_attributes = True



from typing import Union
from urllib.parse import urlencode

from a360_services.utils.service_provider import ServiceProvider
from a360_services.utils.services_list import Services


class DictionaryService:
    def __init__(self, service_provider: ServiceProvider):
        self.service_provider = service_provider

    def get_allergies(self, icd10_codes: list[str]) -> dict:
        query_params = {'icd10_code': icd10_codes}
        request_path = f"/allergies?{urlencode(query_params, doseq=True)}"
        return self.service_provider.fetch_data(
            service=Services.dict,
            request_path=request_path
        ).get('items')

    def get_medical_conditions(self, icd10_codes: list[str]) -> dict:
        query_params = {'icd10_code': icd10_codes}
        request_path = f"/medical_conditions?{urlencode(query_params, doseq=True)}"
        return self.service_provider.fetch_data(
            service=Services.dict,
            request_path=request_path
        ).get('items')

    def get_countries(self, iso_codes: list[str]) -> dict:
        query_params = {'iso_code': iso_codes}
        request_path = f"/countries?{urlencode(query_params, doseq=True)}"
        return self.service_provider.fetch_data(
            service=Services.dict,
            request_path=request_path
        ).get('items')

    def get_states(self,
                   iso_codes: list[str],
                   country_iso_code: Union[None,
                                           str] = None) -> dict:
        query_params = {'iso_code': iso_codes}
        request_path = f"/states?{urlencode(query_params, doseq=True)}"
        if country_iso_code:
            request_path = f"/countries/{country_iso_code}/states?{urlencode(query_params, doseq=True)}"
        return self.service_provider.fetch_data(
            service=Services.dict,
            request_path=request_path
        ).get('items')

    def get_cities(self, city_ids: list[str]) -> dict:
        query_params = {'id': city_ids}
        request_path = f"/cities?{urlencode(query_params, doseq=True)}"
        return self.service_provider.fetch_data(
            service=Services.dict,
            request_path=request_path
        ).get('items')

    def get_tag_categories(
            self,
            search: str = None,
            is_core: bool = None,
            is_active: bool = None) -> dict:
        query_params = {}

        if search is not None:
            query_params['search'] = search
        if is_core is not None:
            query_params['is_core'] = is_core
        if is_active is not None:
            query_params['is_active'] = is_active

        request_path = f"/tag_categories?{urlencode(query_params, doseq=True)}"

        return self.service_provider.fetch_data(
            service=Services.dict,
            request_path=request_path
        )

    def get_tag_category(self, tag_category_id: str) -> dict:
        request_path = f"/tag_categories/{tag_category_id}"
        return self.service_provider.fetch_data(
            service=Services.dict,
            request_path=request_path
        )

    def get_tags(self, search: str = None, is_active: bool = None) -> dict:
        query_params = {}

        if search is not None:
            query_params['search'] = search
        if is_active is not None:
            query_params['is_active'] = is_active

        request_path = f"/tags?{urlencode(query_params, doseq=True)}"

        return self.service_provider.fetch_data(
            service=Services.dict,
            request_path=request_path
        )

    def get_tag(self, tag_id: str) -> dict:
        request_path = f"/tags/{tag_id}"
        return self.service_provider.fetch_data(
            service=Services.dict,
            request_path=request_path
        )



import os
from pydantic import BaseModel


class Settings(BaseModel):
    PROJECT_BASE_URL: str = os.getenv('PROJECT_BASE_URL', 'localhost:8090')
    PROJECT_HOST_SCHEME: str = os.getenv('PROJECT_HOST_SCHEME', 'http')

    AWS_SES_REGION: str = os.getenv('AWS_SES_REGION')
    AWS_SES_SOURCE_EMAIL: str = os.getenv('AWS_SES_SOURCE_EMAIL')


settings = Settings()



from enum import Enum


class Role(str, Enum):
    ADMIN = "ROLE_ADMIN",
    SERVICE = "ROLE_SERVICE",
    AI_TESTER = "ROLE_AI_TESTER",
    PRACTICE_ADMIN = "ROLE_PRACTICE_ADMIN",
    PRACTICE_OFFICE_ADMIN = "ROLE_PRACTICE_OFFICE_ADMIN",
    PRACTICE_DOCTOR = "ROLE_PRACTICE_DOCTOR",
    PRACTICE_SECRETARY = "ROLE_PRACTICE_SECRETARY",



import httpx
from fastapi import Request
from fastapi.security.utils import get_authorization_scheme_param

from ..settings import settings
from .services_list import Services


class ServiceProvider:
    def __init__(self, client: httpx.Client, token: str = None):
        self.client = client
        self.token = token

    def fetch_data(self, service: Services, request_path: str) -> dict:
        headers = {}

        if self.token:
            headers["Authorization"] = f"Bearer {self.token}"

        response = self.client.get(
            f"{settings.PROJECT_HOST_SCHEME}://{service.value}/services/{service.name}{request_path}",
            headers=headers)
        response.raise_for_status()
        return response.json()


def get_service_provider(request: Request) -> ServiceProvider:
    authorization: str = request.headers.get("Authorization")
    token = None

    if authorization:
        scheme, param = get_authorization_scheme_param(authorization)
        if scheme.lower() == "bearer":
            token = param

    client = httpx.Client()
    return ServiceProvider(client=client, token=token)



from .services.aws_ses import AWSSesService
from .settings import settings


def get_mail_service() -> AWSSesService:
    return AWSSesService(
        region_name=settings.AWS_SES_REGION,
        source_email=settings.AWS_SES_SOURCE_EMAIL,
    )



import os
from pydantic import BaseModel


class Settings(BaseModel):
    AWS_COGNITO_REGION: str = os.getenv('AWS_COGNITO_REGION')
    AWS_COGNITO_USER_POOL_ID: str = os.getenv('AWS_COGNITO_USER_POOL_ID')
    AWS_COGNITO_APP_CLIENT_ID: str = os.getenv('AWS_COGNITO_APP_CLIENT_ID')
    AWS_COGNITO_APP_CLIENT_SECRET: str = os.getenv(
        'AWS_COGNITO_APP_CLIENT_SECRET')


settings = Settings()



from a360_security.enums.role import Role

ROLE_HIERARCHY = {
    Role.PRACTICE_SECRETARY: [],
    Role.PRACTICE_DOCTOR: [
        Role.PRACTICE_SECRETARY.value,
    ],
    Role.PRACTICE_OFFICE_ADMIN: [
        Role.PRACTICE_DOCTOR.value,
        Role.PRACTICE_SECRETARY.value,
    ],
    Role.PRACTICE_ADMIN: [
        Role.PRACTICE_DOCTOR.value,
        Role.PRACTICE_OFFICE_ADMIN.value,
        Role.PRACTICE_SECRETARY.value,
    ],
    Role.AI_TESTER: [],
    Role.SERVICE: [],
    Role.ADMIN: [
        Role.SERVICE.value,
        Role.AI_TESTER.value,
        Role.PRACTICE_ADMIN.value,
        Role.PRACTICE_OFFICE_ADMIN.value,
        Role.PRACTICE_DOCTOR.value,
        Role.PRACTICE_SECRETARY.value,
    ],
}

ROLES_PRACTICE = [
    Role.PRACTICE_SECRETARY,
    Role.PRACTICE_DOCTOR,
    Role.PRACTICE_OFFICE_ADMIN,
    Role.PRACTICE_ADMIN,
]


def has_role(user_roles: list[str], required_role: Role) -> bool:
    all_roles = set(user_roles)
    roles_to_check = list(user_roles)
    while roles_to_check:
        role = roles_to_check.pop()
        if role in ROLE_HIERARCHY:
            inherited_roles = ROLE_HIERARCHY[role]
            for inherited_role in inherited_roles:
                if inherited_role not in all_roles:
                    all_roles.add(inherited_role)
                    roles_to_check.append(inherited_role)

    return required_role.value in all_roles



import os
from pydantic import BaseModel


class Settings(BaseModel):
    PROJECT_BASE_URL: str = os.getenv('PROJECT_BASE_URL', 'localhost:8090')
    PROJECT_HOST_SCHEME: str = os.getenv('PROJECT_HOST_SCHEME', 'http')
    PROJECT_HOST_SERVICE_DICT: str = os.getenv(
        'PROJECT_HOST_SERVICE_DICT', 'service_dict')
    PROJECT_HOST_SERVICE_PATIENTS: str = os.getenv(
        'PROJECT_HOST_SERVICE_PATIENTS', 'service_patients')
    PROJECT_HOST_SERVICE_PRACTICES: str = os.getenv(
        'PROJECT_HOST_SERVICE_PRACTICES', 'service_practices')
    PROJECT_HOST_SERVICE_CONSULTATIONS: str = os.getenv(
        'PROJECT_HOST_SERVICE_CONSULTATIONS', 'service_consultations')
    PROJECT_HOST_SERVICE_PRODUCTS: str = os.getenv(
        'PROJECT_HOST_SERVICE_PRODUCTS', 'service_products')
    PROJECT_HOST_SERVICE_ML: str = os.getenv(
        'PROJECT_HOST_SERVICE_ML', 'service_ml')
    PROJECT_HOST_SERVICE_CMS: str = os.getenv(
        'PROJECT_HOST_SERVICE_CMS', 'service_cms')


settings = Settings()



import uuid
from typing import Union

from a360_services.utils.service_provider import ServiceProvider
from a360_services.utils.services_list import Services


class PracticeService:
    def __init__(self, service_provider: ServiceProvider):
        self.service_provider = service_provider

    def get_expert(
            self,
            expert_id: Union[uuid.UUID, None] = None,
            user_id: Union[uuid.UUID, None] = None
    ) -> dict:
        if expert_id is None and user_id is None:
            raise ValueError("Either expert_id or user_id must be provided")
        if expert_id is not None and user_id is not None:
            raise ValueError(
                "Only one of expert_id or user_id should be provided")

        if expert_id is not None:
            request_path = f"/experts/{str(expert_id)}"
        else:
            request_path = f"/experts/by_user_id/{str(user_id)}"

        return self.service_provider.fetch_data(
            service=Services.practice,
            request_path=request_path
        )



import re
import html


def html_to_text(html_content: str) -> str:
    clean_text = re.sub('<[^<]+?>', '', html_content)
    clean_text = html.unescape(clean_text)
    clean_text = re.sub(r'\s+', ' ', clean_text).strip()

    return clean_text



import boto3
from typing import Union
from ..utils.email_utils import html_to_text


class AWSSesService:
    def __init__(self, region_name: str, source_email: str):
        self.client = boto3.client('ses', region_name=region_name)
        self.source_email = source_email

    def send_email(
            self,
            recipient_name: str,
            recipient_address: str,
            subject: str,
            html_content: str,
            text_content: Union[str, None] = None
    ):
        if text_content is None:
            text_content = html_to_text(html_content)

        response = self.client.send_email(
            Source=self.source_email,
            Destination={
                'ToAddresses': [f"{recipient_name} <{recipient_address}>"]
            },
            Message={
                'Subject': {
                    'Data': subject
                },
                'Body': {
                    'Text': {
                        'Data': text_content
                    },
                    'Html': {
                        'Data': html_content
                    }
                }
            }
        )
        return response



from fastapi import Depends
from fastapi.security import HTTPBearer, HTTPAuthorizationCredentials

security = HTTPBearer()


def get_token(
        credentials: HTTPAuthorizationCredentials = Depends(security)) -> str:
    return credentials.credentials
